@@ -11,6 +11,8 @@
 #define FILTER1 "src port > 80"
 #define FILTER2 "in if 2"
 
+#define LLUI long long unsigned int
+
 int main(int argc, char **argv) {
 
 	lnf_file_t *filep;
@@ -58,16 +60,20 @@ int main(int argc, char **argv) {
 	lnf_rec_init(&recp);
 	lnf_mem_init(&memp);
 
+	lnf_mem_fastaggr(memp, LNF_FAST_AGGR_BASIC);
+
 	/* set rules for aggregation srcip/24,srcport,dstas */
 	lnf_mem_fadd(memp, LNF_FLD_SRCADDR, LNF_AGGR_KEY|LNF_SORT_DESC, 24, 64);
 	lnf_mem_fadd(memp, LNF_FLD_SRCPORT, LNF_AGGR_KEY, 0, 0); 
 	lnf_mem_fadd(memp, LNF_FLD_DSTAS, LNF_AGGR_KEY, 0, 0);
 
+/*
 	lnf_mem_fadd(memp, LNF_FLD_FIRST, LNF_AGGR_MIN, 0, 0);
 	lnf_mem_fadd(memp, LNF_FLD_LAST, LNF_AGGR_MAX, 0, 0);
-	lnf_mem_fadd(memp, LNF_FLD_TCP_FLAGS, LNF_AGGR_OR, 0, 0);
+//	lnf_mem_fadd(memp, LNF_FLD_TCP_FLAGS, LNF_AGGR_OR, 0, 0);
 	lnf_mem_fadd(memp, LNF_FLD_DOCTETS, LNF_AGGR_SUM, 0, 0);
 	lnf_mem_fadd(memp, LNF_FLD_DPKTS, LNF_AGGR_SUM, 0, 0);
+*/
 
 
 	while (lnf_read(filep, recp) != LNF_EOF) {
@@ -89,7 +95,7 @@ int main(int argc, char **argv) {
 			printf(" %s :%d -> %s :%d %llu %llu %llu\n", 
 					sbuf, brec.srcport, 
 					dbuf, brec.dstport,  
-					brec.first, brec.bytes, brec.pkts);
+					(LLUI)brec.first, (LLUI)brec.bytes, (LLUI)brec.pkts);
 		}
 	}
 
@@ -113,7 +119,7 @@ int main(int argc, char **argv) {
 			printf(" %s :%d -> %s :%d %llu %llu %llu\n", 
 					sbuf, brec.srcport, 
 					dbuf, brec.dstport,  
-					brec.first, brec.bytes, brec.pkts);
+					(LLUI)brec.first, (LLUI)brec.bytes, (LLUI)brec.pkts);
 		}
 	}
 
@@ -122,6 +128,8 @@ int main(int argc, char **argv) {
 	lnf_mem_free(memp);
 	lnf_rec_free(recp);
 	lnf_close(filep);
+
+	return 0;
 }
 
 
