@@ -19,6 +19,17 @@
 #include "bit_array.h"
 #include "hash_table.h"
 
+#ifndef HAVE_HTONLL
+#ifdef WORDS_BIGENDIAN
+#	define ntohll(n)	(n)
+#	define htonll(n)	(n)
+#else
+#	define ntohll(n)	(((uint64_t)ntohl(n)) << 32) + ntohl((n) >> 32)
+#	define htonll(n)	(((uint64_t)htonl(n)) << 32) + htonl((n) >> 32)
+#endif
+#define HAVE_HTONLL 1
+#endif
+
 
 /* list of maps used in file taht we create */
 typedef struct lnf_map_list_s {
@@ -61,8 +72,6 @@ typedef struct lnf_file_s {
 //	bit_array_t				extensions_arr;			/* structure initialised at the beginging to store bitt array of extensions from last record */
 	uint64_t                processed_blocks;
 	uint64_t                skipped_blocks;
-	uint64_t                processed_records;
-	uint64_t                current_processed_blocks;
 	uint64_t                processed_bytes;
 } lnf_file_t;
 
@@ -78,15 +87,21 @@ void lnf_seterror(char *format, ...);
 /* Mem heap section - Aggergations, Sorting                     */
 /****************************************************************/
 
+/* define aggregation function prototype */
+/* gets *a and *b and set result to *a */
+typedef void (*lnf_mem_aggr_func_t)(char *a, char *b);
+
 /* field list for aggregation and sorting */
 typedef struct lnf_fieldlist_s {
 	int field;
 	int size;			/* size of the field ib bytes */
+	int type;			/* field data type */
 	int offset;			/* offset from the beggining of the record */
 	int aggr_flag;
 	int sort_flag;
 	int numbits;
 	int numbits6;
+	lnf_mem_aggr_func_t	aggr_func;	/* function called for aggregation */
 	void *next;
 } lnf_fieldlist_t;
 
@@ -100,12 +115,14 @@ typedef struct lnf_mem_s {
 	int	val_len;
 //	lnf_fieldlist_t *sort_list;		/* list of fields to sort */
 //	int	sort_len;
+	int fastaggr_mode;				/* mode is fast aggergation is enabled */
 	int sort_field;					/* field identification for sorting */
 	int sort_offset;				/* offset in the record field */
 	int sort_flags;					/* search sort field in ket or aggregated value */
 #define LNF_SORT_FLD_NONE 0x0
 #define LNF_SORT_FLD_IN_KEY 0x1
 #define LNF_SORT_FLD_IN_VAL 0x2
+#define LNF_SORT_FLD_IN_FLAGS 0xF
 #ifdef LNF_THREADS
 	pthread_key_t thread_id_key;	/* key for thread specific id */
 	pthread_mutex_t thread_mutex;
@@ -137,8 +154,19 @@ int lnf_mem_fill_buf(lnf_fieldlist_t *fld, lnf_rec_t *rec, char *buf);
 void lnf_mem_fill_rec(lnf_fieldlist_t *fld, char *buf, lnf_rec_t *rec);
 
 
-int lnf_filedlist_add(lnf_fieldlist_t **list, lnf_fieldlist_t *snode, int *sizep, int maxsize, int *offset);
+int lnf_filedlist_add_or_upd(lnf_fieldlist_t **list, lnf_fieldlist_t *snode, int *sizep, int maxsize, int *offset);
+void lnf_mem_fastaggr_callback(char *key, char *hval, char *uval, void *lnf_mem);
 void lnf_mem_aggr_callback(char *key, char *hval, char *uval, void *lnf_mem);
 int lnf_mem_sort_callback(char *key1, char *val1, char *key2, char *val2, void *p);
 int lnf_mem_done(lnf_mem_t *lnf_mem);
 
+
+/* structure for fast aggregation support */
+typedef struct lnf_fastaggr_s {
+	uint64_t	first;
+	uint64_t	last;
+	uint64_t	doctets;
+	uint64_t	dpkts;
+	uint64_t	aggr_flows;
+} lnf_fastaggr_t;
+
